﻿/*
Автор: Veretragna
Скрипт создавался для быстрого экспорта отдельных страниц документа InDesign в форматы JPG и PNG с возможностью именования страниц, одновременного экспорта в оба формата, указания размеров страницы в имени файла и одновременного экспорта с несколькими DPI.
Скрипт часто используется и должен выполнять работу быстро и молча, поэтому графический интерфейс не планируется.
Заложена возможность провести эту операцию для всех открытых документов (см. строку 37).

Инструкция по использованию: 
* Выделить нужную страницу документа с помощью инструмента Page Tool и подписать страницу в панели Script Label - это будет именем страницы при экспорте (повторить необходимое количество раз);
* Запустить скрипт.

В папке документа InDesign будет создана папка с тем же именем
(c:\projects\layout.indd -> c:\projects\layout\), 
и в нее будут экспортированы страницы документа поименно.
Пример имени файла после экспорта: "Страница такая-то - 120x150mm (300dpi).png"

Настройки экспорта прописываются в скрипте, наиболее важные настройки выведены в начало скрипта.
Пользуйтесь на здоровье.
*/

polyfillForEach(); // Mozilla polyfill adding .forEach() method to plain JS arrays

var aDPI = [300, 72, 44]; // DPI list
var aFormat = [[ExportFormat.JPG, '.jpg'], [ExportFormat.PNG_FORMAT, '.png']]; // formats and extensions (2d array)
var bTransparentBackground = false; // for PNG only; true|false
var bUseDocumentBleeds = false; // use document bleeds; true|false
var bOverwrite = true; // overwrite existing files; true|false

/*
var aDPI = [300];
var aFormat = [[ExportFormat.PNG_FORMAT, '.png']];
*/

// perform this task for active document
exportImagesFromDoc(app.activeDocument); alert('Done.');

// perform this task for all open docs
//app.documents.everyItem().getElements().forEach(function (obj) { exportImagesFromDoc(obj) }); alert('Done.'); // getting items as an array, not object

function exportImagesFromDoc(doc) {
	set_export_options();
	
	var exportPath = doc.fullName.toString().substr(0, doc.fullName.toString().lastIndexOf('.'));
	var w = new Window ("palette", "Progress Bar");
	var progress = progress_bar (w, aDPI.length * aFormat.length * doc.pages.length);
	var measurementUnits = getUnits();
	
	aDPI.forEach( function (dpi) {
		set_dpi(dpi);
		doc.pages.everyItem().getElements().forEach( function (page) { // getting pages as an array, not object
			if (page.label) {
				set_page_string(page.name);
				aFormat.forEach( function (format) {
					var pageBounds = page.bounds;
					var fullName = exportPath + '/' +
						page.label +
						" - " + Math.round(pageBounds[3]-pageBounds[1]) + "x" + Math.round(pageBounds[2]-pageBounds[0]) + measurementUnits +
						" (" + dpi + "dpi)" +
						format[1]; // can be split with +"/"+ to create subfolders
					
					var fullPath = fullName.substr(0, fullName.lastIndexOf('/')); // reduce fullname to directory
					var f = new Folder(fullPath);
					if (!f.exists)
						f.create();
					
					var fullFile = new File(fullName);
					if (!fullFile.exists || bOverwrite)
					{
						doc.exportFile(format[0], fullFile);
					}
					progress.value++;
				});
			}
		});
	});
	
	progress.parent.close();
}

function progress_bar (w, stop)
{
	var pbar = w.add ("progressbar", undefined, 1, stop);
	pbar.preferredSize = [300,20];
	w.show ();
	return pbar;
}

function set_export_options()
{ 
	with(app.pngExportPreferences)
	{ 
		PNGExportRange = PNGExportRangeEnum.EXPORT_RANGE;
		antiAlias = true; 
		embedColorProfile = false;
		exportingSpread = false; 
		PNGColorSpace = PNGColorSpaceEnum.RGB;
		PNGQuality = PNGQualityEnum.MAXIMUM;
		simulateOverprint = false; 
		transparentBackground = bTransparentBackground; 
		useDocumentBleeds = bUseDocumentBleeds; 
	}
	
	with(app.jpegExportPreferences)
	{ 
		jpegExportRange = ExportRangeOrAllPages.EXPORT_RANGE;
		antiAlias = true; 
		embedColorProfile = false;
		exportingSpread = false; 
		jpegColorSpace = JpegColorSpaceEnum.RGB;
		//jpegColorSpace = JpegColorSpaceEnum.CMYK;
		//jpegColorSpace = JpegColorSpaceEnum.GRAY;
		jpegQuality = JPEGOptionsQuality.HIGH;
		simulateOverprint = false;
		useDocumentBleeds = bUseDocumentBleeds;
	}
}

function set_dpi(vdpi)
{
	app.pngExportPreferences.exportResolution = vdpi;
	app.jpegExportPreferences.exportResolution = vdpi;
}

function set_page_string(epage)
{
	app.pngExportPreferences.pageString = epage;
	app.jpegExportPreferences.pageString = epage;
}

function getUnits() {  
	var ret;
	switch (app.activeDocument.viewPreferences.horizontalMeasurementUnits) {
		case MeasurementUnits.MILLIMETERS : ret = 'mm'; break;
		case MeasurementUnits.CENTIMETERS : ret = 'cm'; break;
		case MeasurementUnits.POINTS : ret = 'pt'; break;
		case MeasurementUnits.PIXELS : ret = 'px'; break;
		case MeasurementUnits.PICAS : ret = 'pc'; break;
		case MeasurementUnits.INCHES : ret = 'in';
	}
	return ret;  
}

function polyfillForEach () { // Mozilla minified polyfill adding .forEach() method to plain JS arrays
	if(!Array.prototype.forEach){Array.prototype.forEach=function(callback,thisArg){var T,k;if(this==null){throw new TypeError(' this is null or not defined');}
	var O=Object(this);var len=O.length>>>0;if(typeof callback!=='function'){throw new TypeError(callback+' is not a function');}
	if(arguments.length>1){T=thisArg;}
	k=0;while(k<len){var kValue;if(k in O){kValue=O[k];callback.call(T,kValue,k,O);}
	k++;}};}
}